@@ -97,7 +97,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.cc 2839 2005-09-07 08:26:45Z fyodor $ */
+/* $Id: NmapOps.cc 2968 2005-12-06 22:26:00Z fyodor $ */
 #include "nmap.h"
 #include "nbase.h"
 #include "NmapOps.h"
@@ -163,7 +163,7 @@ const struct in_addr *NmapOps::v4sourceip() {
 }
 
 // Number of milliseconds since getStartTime().  The current time is an
-// optional argument to avoid an extre gettimeofday() call.
+// optional argument to avoid an extra gettimeofday() call.
 int NmapOps::TimeSinceStartMS(struct timeval *now) {
   struct timeval tv;
   if (!now)
@@ -209,7 +209,7 @@ void NmapOps::Initialize() {
   min_rtt_timeout = MIN_RTT_TIMEOUT;
   initial_rtt_timeout = INITIAL_RTT_TIMEOUT;
   min_host_group_sz = 1;
-  max_host_group_sz = 100000; // don't want to be restrictive unles user sets
+  max_host_group_sz = 100000; // don't want to be restrictive unless user sets
   max_tcp_scan_delay = MAX_TCP_SCAN_DELAY;
   max_udp_scan_delay = MAX_UDP_SCAN_DELAY;
   max_ips_to_scan = 0;
@@ -291,6 +291,21 @@ void NmapOps::ValidateOptions() {
     //    if (verbose) error("No tcp, udp, or ICMP scantype specified, assuming %s scan. Use -sP if you really don't want to portscan (and just want to see what hosts are up).", synscan? "SYN Stealth" : "vanilla tcp connect()");
   }
 
+  if ((pingtype & PINGTYPE_TCP) && (!o.isr00t || o.af() != AF_INET)) {
+    /* We will have to do a connect() style ping */
+    if (num_ping_synprobes && num_ping_ackprobes) {
+      fatal("Cannot use both SYN and ACK ping probes if you are nonroot or using IPv6");
+    }
+    
+    if (num_ping_ackprobes > 0) { 
+      memcpy(ping_synprobes, ping_ackprobes, num_ping_ackprobes * sizeof(*ping_synprobes));
+      num_ping_synprobes = num_ping_ackprobes;
+      num_ping_ackprobes = 0;
+    }
+    pingtype &= ~PINGTYPE_TCP_USE_ACK;
+    pingtype |= PINGTYPE_TCP_USE_SYN;
+  }
+
   if (pingtype != PINGTYPE_NONE && spoofsource) {
     error("WARNING:  If -S is being used to fake your source address, you may also have to use -e <iface> and -P0 .  If you are using it to specify your real source address, you can ignore this warning.");
   }
@@ -312,18 +327,6 @@ void NmapOps::ValidateOptions() {
    fatal("Sorry, UDP Ping (-PU) only works if you are root (because we need to read raw responses off the wire) and only for IPv4 (cause fyodor is too lazy right now to add IPv6 support and nobody has sent a patch)");
  }
 
- if ((pingtype & PINGTYPE_TCP) && (!o.isr00t || o.af() != AF_INET)) {
-   /* We will have to do a connect() style ping */
-   if (num_ping_synprobes && num_ping_ackprobes) {
-     fatal("Cannot use both SYN and ACK ping probes if you are nonroot or using IPv6");
-   }
-
-   if (num_ping_ackprobes > 0) { 
-     memcpy(ping_synprobes, ping_ackprobes, num_ping_ackprobes * sizeof(*ping_synprobes));
-     num_ping_synprobes = num_ping_ackprobes;
-     num_ping_ackprobes = 0;
-   }
- }
 
  if (ipprotscan + (TCPScan() || UDPScan()) + listscan + pingscan > 1) {
    fatal("Sorry, the IPProtoscan, Listscan, and Pingscan (-sO, -sL, -sP) must currently be used alone rather than combined with other scan types.");
@@ -436,7 +439,7 @@ void NmapOps::setMaxRttTimeout(int rtt)
 
 void NmapOps::setMinRttTimeout(int rtt) 
 { 
-  if (rtt < 0) fatal("NmapOps::setMaxRttTimeout(): minimum round trip time must be at least 0");
+  if (rtt < 0) fatal("NmapOps::setMinRttTimeout(): minimum round trip time must be at least 0");
   min_rtt_timeout = rtt; 
   if (rtt > max_rtt_timeout) max_rtt_timeout = rtt;  
   if (rtt > initial_rtt_timeout) initial_rtt_timeout = rtt;
@@ -444,7 +447,7 @@ void NmapOps::setMinRttTimeout(int rtt)
 
 void NmapOps::setInitialRttTimeout(int rtt) 
 { 
-  if (rtt <= 0) fatal("NmapOps::setMaxRttTimeout(): initial round trip time must be greater than 0");
+  if (rtt <= 0) fatal("NmapOps::setInitialRttTimeout(): initial round trip time must be greater than 0");
   initial_rtt_timeout = rtt; 
   if (rtt > max_rtt_timeout) max_rtt_timeout = rtt;  
   if (rtt < min_rtt_timeout) min_rtt_timeout = rtt;
