@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap.cc 2868 2005-09-12 08:46:04Z fyodor $ */
+/* $Id: nmap.cc 2968 2005-12-06 22:26:00Z fyodor $ */
 
 #include "nmap.h"
 #include "osscan.h"
@@ -160,7 +160,7 @@ static int parse_scanflags(char *arg) {
 static int parse_bounce_argument(struct ftpinfo *ftp, char *url) {
   char *p = url,*q, *s;
 
-  if ((q = strrchr(url, '@'))) /*we have username and/or pass */ {
+  if ((q = strrchr(url, '@'))) { /*we have username and/or pass */
     *(q++) = '\0';
     if ((s = strchr(q, ':')))
       { /* has portno */
@@ -278,14 +278,15 @@ int nmap_main(int argc, char *argv[]) {
       {"randomize_hosts", no_argument, 0, 0},
       {"osscan_limit", no_argument, 0, 0}, /* skip OSScan if no open ports */
       {"osscan_guess", no_argument, 0, 0}, /* More guessing flexability */
+      {"fuzzy", no_argument, 0, 0}, /* Alias for osscan_guess */
       {"packet_trace", no_argument, 0, 0}, /* Display all packets sent/rcv */
       {"version_trace", no_argument, 0, 0}, /* Display -sV related activity */
-      {"fuzzy", no_argument, 0, 0}, /* Alias for osscan_guess */
       {"data_length", required_argument, 0, 0},
       {"send_eth", no_argument, 0, 0},
       {"send_ip", no_argument, 0, 0},
       {"stylesheet", required_argument, 0, 0},
       {"no_stylesheet", no_argument, 0, 0},
+      {"webxml", no_argument, 0, 0},
       {"rH", no_argument, 0, 0},
       {"vv", no_argument, 0, 0},
       {"ff", no_argument, 0, 0},
@@ -294,6 +295,7 @@ int nmap_main(int argc, char *argv[]) {
       {"append_output", no_argument, 0, 0},
       {"noninteractive", no_argument, 0, 0},
       {"spoof_mac", required_argument, 0, 0},
+      {"thc", no_argument, 0, 0},  
       {"ttl", required_argument, 0, 0}, /* Time to live */
       {"allports", no_argument, 0, 0},
       {"version_intensity", required_argument, 0, 0},
@@ -309,14 +311,17 @@ int nmap_main(int argc, char *argv[]) {
   }
   fakeargv[argc] = NULL;
 
-  emptystring[0] = '\0'; /* It wouldn't be an emptystring w/o this ;) */
+  emptystring[0] = '\0'; /* It wouldn't be an empty string w/o this ;) */
 
   if (argc < 2 ) printusage(argv[0], -1);
   Targets.reserve(100);
+#ifdef WIN32
+  win_pre_init();
+#endif
 
   /* OK, lets parse these args! */
   optind = 1; /* so it can be called multiple times */
-  while((arg = getopt_long_only(argc,fakeargv,"6Ab:D:d::e:Ffg:hIi:M:m:NnOo:P:p:qRrS:s:T:Vv", long_options, &option_index)) != EOF) {
+  while((arg = getopt_long_only(argc,fakeargv,"6Ab:D:d::e:Ffg:hIi:M:m:nOo:P:p:qRrS:s:T:Vv", long_options, &option_index)) != EOF) {
     switch(arg) {
     case 0:
       if (strcmp(long_options[option_index].name, "max_rtt_timeout") == 0) {
@@ -443,6 +448,8 @@ int nmap_main(int argc, char *argv[]) {
 	o.setXSLStyleSheet(optarg);
       } else if (strcmp(long_options[option_index].name, "no_stylesheet") == 0) {
 	o.setXSLStyleSheet(NULL);
+      } else if (strcmp(long_options[option_index].name, "webxml") == 0) {
+	o.setXSLStyleSheet("http://www.insecure.org/nmap/data/nmap.xsl");
       } else if (strcmp(long_options[option_index].name, "oN") == 0) {
 	normalfilename = optarg;
       } else if (strcmp(long_options[option_index].name, "oG") == 0 ||
@@ -451,7 +458,7 @@ int nmap_main(int argc, char *argv[]) {
       } else if (strcmp(long_options[option_index].name, "oS") == 0) {
 	kiddiefilename = optarg;
       } else if (strcmp(long_options[option_index].name, "oH") == 0) {
-	fatal("HTML output is not yet supported");
+	fatal("HTML output is not directly supported, though Nmap includes an XSL for transforming XML output into HTML.  See the man page.");
       } else if (strcmp(long_options[option_index].name, "oX") == 0) {
 	xmlfilename = optarg;
       } else if (strcmp(long_options[option_index].name, "oA") == 0) {
@@ -462,6 +469,9 @@ int nmap_main(int argc, char *argv[]) {
 	machinefilename = strdup(buf);
 	snprintf(buf, sizeof(buf), "%s.xml", optarg);
 	xmlfilename = strdup(buf);
+      } else if (strcmp(long_options[option_index].name, "thc") == 0) {
+	printf("!!Greets to Van Hauser, Plasmoid, Skyper and the rest of THC!!\n");
+	exit(0);
       }
       else if (strcmp(long_options[option_index].name, "iL") == 0) {
 	if (inputfd) {
@@ -532,7 +542,7 @@ int nmap_main(int argc, char *argv[]) {
 	  if (resolve(p, &o.decoys[o.numdecoys])) {
 	    o.numdecoys++;
 	  } else {
-	    fatal("Failed to resolve decoy host: %s (must be hostname or IP address", optarg);
+	    fatal("Failed to resolve decoy host: %s (must be hostname or IP address)", optarg);
 	  }
 	}
 	if (q) {
@@ -619,6 +629,8 @@ int nmap_main(int argc, char *argv[]) {
 	}
       }
       else if (*optarg == 'T' || *optarg == 'A') {
+	/* NmapOps::ValidateOptions() takes care of changing this
+	   to SYN if not root or if IPv6 */
 	o.pingtype |= (PINGTYPE_TCP|PINGTYPE_TCP_USE_ACK);
 	if (isdigit((int) *(optarg+1))) {
 	  o.num_ping_ackprobes = numberlist2array(optarg+1, o.ping_ackprobes, sizeof(o.ping_ackprobes), &proberr);
@@ -1558,32 +1570,87 @@ struct scan_lists *getpts(char *origexpr) {
 }
 
 void printusage(char *name, int rc) {
-  printf(
-	 "Nmap %s Usage: nmap [Scan Type(s)] [Options] <host or net list>\n"
-	 "Some Common Scan Types ('*' options require root privileges)\n"
-	 "* -sS TCP SYN stealth port scan (default if privileged (root))\n"
-	 "  -sT TCP connect() port scan (default for unprivileged users)\n"
-	 "* -sU UDP port scan\n"
-	 "  -sP ping scan (Find any reachable machines)\n"
-	 "* -sF,-sX,-sN Stealth FIN, Xmas, or Null scan (experts only)\n"
-         "  -sV Version scan probes open ports determining service & app names/versions\n"
-	 "  -sR RPC scan (use with other scan types)\n"
-	 "Some Common Options (none are required, most can be combined):\n"
-	 "* -O Use TCP/IP fingerprinting to guess remote operating system\n"
-	 "  -p <range> ports to scan.  Example range: 1-1024,1080,6666,31337\n"
-	 "  -F Only scans ports listed in nmap-services\n"
-	 "  -v Verbose. Its use is recommended.  Use twice for greater effect.\n"
-	 "  -P0 Don't ping hosts (needed to scan www.microsoft.com and others)\n"
-	 "* -Ddecoy_host1,decoy2[,...] Hide scan using many decoys\n"
-         "  -6 scans via IPv6 rather than IPv4\n"
-	 "  -T <Paranoid|Sneaky|Polite|Normal|Aggressive|Insane> General timing policy\n"
-	 "  -n/-R Never do DNS resolution/Always resolve [default: sometimes resolve]\n"
-	 "  -oN/-oX/-oG <logfile> Output normal/XML/grepable scan logs to <logfile>\n"
-	 "  -iL <inputfile> Get targets from file; Use '-' for stdin\n"
-	 "* -S <your_IP>/-e <devicename> Specify source address or network interface\n"
-	 "  --interactive Go into interactive mode (then press h for help)\n"
-	 "Example: nmap -v -sS -O www.my.com 192.168.0.0/16 '192.88-90.*.*'\n"
-	 "SEE THE MAN PAGE FOR MANY MORE OPTIONS, DESCRIPTIONS, AND EXAMPLES \n", NMAP_VERSION);
+
+printf("%s %s ( %s )\n"
+       "Usage: nmap [Scan Type(s)] [Options] {target specification}\n"
+       "TARGET SPECIFICATION:\n"
+       "  Can pass hostnames, IP addresses, networks, etc.\n"
+       "  Ex: scanme.nmap.org, microsoft.com/24, 192.168.0.1; 10.0.0-255.1-254\n"
+       "  -iL <inputfilename>: Input from list of hosts/networks\n"
+       "  -iR <num hosts>: Choose random targets\n"
+       "  --exclude <host1[,host2][,host3],...>: Exclude hosts/networks\n"
+       "  --excludefile <exclude_file>: Exclude list from file\n"
+       "HOST DISCOVERY:\n"
+       "  -sL: List Scan - simply list targets to scan\n"
+       "  -sP: Ping Scan - go no further than determining if host is online\n"
+       "  -P0: Treat all hosts as online -- skip host discovery\n"
+       "  -PS/PA/PU [portlist]: TCP SYN/ACK or UDP discovery to given ports\n"
+       "  -PE/PP/PM: ICMP echo, timestamp, and netmask request discovery probes\n"
+       "  -n/-R: Never do DNS resolution/Always resolve [default: sometimes]\n"
+       "SCAN TECHNIQUES:\n"
+       "  -sS/sT/sA/sW/sM: TCP SYN/Connect()/ACK/Window/Maimon scans\n"
+       "  -sN/sF/sX: TCP Null, FIN, and Xmas scans\n"
+       "  --scanflags <flags>: Customize TCP scan flags\n"
+       "  -sI <zombie host[:probeport]>: Idlescan\n"
+       "  -sO: IP protocol scan\n"
+       "  -b <ftp relay host>: FTP bounce scan\n"
+       "PORT SPECIFICATION AND SCAN ORDER:\n"
+       "  -p <port ranges>: Only scan specified ports\n"
+       "    Ex: -p22; -p1-65535; -p U:53,111,137,T:21-25,80,139,8080\n"
+       "  -F: Fast - Scan only the ports listed in the nmap-services file)\n"
+       "  -r: Scan ports consecutively - don't randomize\n"
+       "SERVICE/VERSION DETECTION:\n"
+       "  -sV: Probe open ports to determine service/version info\n"
+       "  --version_light: Limit to most likely probes for faster identification\n"
+       "  --version_all: Try every single probe for version detection\n"
+       "  --version_trace: Show detailed version scan activity (for debugging)\n"
+       "OS DETECTION:\n"
+       "  -O: Enable OS detection\n"
+       "  --osscan_limit: Limit OS detection to promising targets\n"
+       "  --osscan_guess: Guess OS more aggressively\n"
+       "TIMING AND PERFORMANCE:\n"
+       "  -T[0-5]: Set timing template (higher is faster)\n"
+       "  --min_hostgroup/max_hostgroup <msec>: Parallel host scan group sizes\n"
+       "  --min_parallelism/max_parallelism <msec>: Probe parallelization\n"
+       "  --min_rtt_timeout/max_rtt_timeout/initial_rtt_timeout <msec>: Specifies\n"
+       "      probe round trip time.\n"
+       "  --host_timeout <msec>: Give up on target after this long\n"
+       "  --scan_delay/--max_scan_delay <msec>: Adjust delay between probes\n"
+       "FIREWALL/IDS EVASION AND SPOOFING:\n"
+       "  -f; --mtu <val>: fragment packets (optionally w/given MTU)\n"
+       "  -D <decoy1,decoy2[,ME],...>: Cloak a scan with decoys\n"
+       "  -S <IP_Address>: Spoof source address\n"
+       "  -e <iface>: Use specified interface\n"
+       "  -g/--source_port <portnum>: Use given port number\n"
+       "  --data_length <num>: Append random data to sent packets\n"
+       "  --ttl <val>: Set IP time-to-live field\n"
+       "  --spoof_mac <mac address/prefix/vendor name>: Spoof your MAC address\n"
+       "OUTPUT:\n"
+       "  -oN/-oX/-oS/-oG <file>: Output scan in normal, XML, s|<rIpt kIddi3,\n"
+       "     and Grepable format, respectively, to the given filename.\n"
+       "  -oA <basename>: Output in the three major formats at once\n"
+       "  -v: Increase verbosity level (use twice for more effect)\n"
+       "  -d[level]: Set or increase debugging level (Up to 9 is meaningful)\n"
+       "  --packet_trace: Show all packets sent and received\n"
+       "  --iflist: Print host interfaces and routes (for debugging)\n"
+       "  --append_output: Append to rather than clobber specified output files\n"
+       "  --resume <filename>: Resume an aborted scan\n"
+       "  --stylesheet <path/URL>: XSL stylesheet to transform XML output to HTML\n"
+       "  --webxml: Reference stylesheet from Insecure.Org for more portable XML\n"
+       "  --no_stylesheet: Prevent associating of XSL stylesheet w/XML output\n"
+       "MISC:\n"
+       "  -6: Enable IPv6 scanning\n"
+       "  -A: Enables OS detection and Version detection\n"
+       "  --datadir <dirname>: Specify custom Nmap data file location\n"
+       "  --send_eth/--send_ip: Send using raw ethernet frames or IP packets\n"
+       "  --privileged: Assume that the user is fully privileged\n"
+       "  -V: Print version number\n"
+       "  -h: Print this help summary page.\n"
+       "EXAMPLES:\n"
+       "  nmap -v -A scanme.nmap.org\n"
+       "  nmap -v -sP 192.168.0.0/16 10.0.0.0/8\n"
+       "  nmap -v -iR 10000 -P0 -p 80\n"
+       "SEE THE MAN PAGE FOR MANY MORE OPTIONS, DESCRIPTIONS, AND EXAMPLES\n", NMAP_NAME, NMAP_VERSION, NMAP_URL);
   exit(rc);
 }
 
@@ -1828,7 +1895,7 @@ char *grab_next_host_spec(FILE *inputfd, int argc, char **fakeargv) {
   } else { 
     host_spec_index = 0;
     while((ch = getc(inputfd)) != EOF) {
-      if (ch == ' ' || ch == '\n' || ch == '\t' || ch == '\0') {
+      if (ch == ' ' || ch == '\r' || ch == '\n' || ch == '\t' || ch == '\0') {
 	if (host_spec_index == 0) continue;
 	host_spec[host_spec_index] = '\0';
 	return host_spec;
